@@ -22,6 +22,7 @@ import logging
 import linecache
 import os
 import sys
+import collections
 from collections import Counter
 from io import open
 from multiprocessing import Pool, cpu_count
@@ -30,7 +31,10 @@ import torch
 import torch.nn as nn
 from scipy.stats import pearsonr, spearmanr
 from sklearn.metrics import f1_score, matthews_corrcoef
+
 from torch.utils.data import Dataset
+from datasets import load_dataset
+from datasets import Dataset as HFDataset
 from tqdm.auto import tqdm
 
 try:
@@ -52,7 +56,9 @@ logger = logging.getLogger(__name__)
 class InputExample(object):
     """A single training/test example for simple sequence classification."""
 
-    def __init__(self, guid, text_a, text_b=None, label=None, x0=None, y0=None, x1=None, y1=None):
+    def __init__(
+        self, guid, text_a, text_b=None, label=None, x0=None, y0=None, x1=None, y1=None
+    ):
         """
         Constructs a InputExample.
 
@@ -89,51 +95,48 @@ class InputFeatures(object):
 
 
 def preprocess_data_multiprocessing(data):
-    examples, tokenizer, max_seq_length = data
-    text = []
-    labels = []
-    if examples[0].text_b is not None:
-        for example in examples:
-            text.append((example.text_a, example.text_b))
-            labels.append(example.label)
-    else:
-        for example in examples:
-            text.append(example.text_a)
-            labels.append(example.label)
+    text_a, text_b, tokenizer, max_seq_length = data
 
-    examples = tokenizer.batch_encode_plus(
-        batch_text_or_text_pairs=text,
+    examples = tokenizer(
+        text=text_a,
+        text_pair=text_b,
         max_length=max_seq_length,
         truncation=True,
         padding="max_length",
         return_tensors="pt",
     )
 
-    return [examples, labels]
+    return examples
 
 
-def preprocess_data(examples, tokenizer, max_seq_length):
-    text = []
-    labels = []
-    if examples[0].text_b is not None:
-        for example in examples:
-            text.append((example.text_a, example.text_b))
-            labels.append(example.label)
+def preprocess_batch_for_hf_dataset(dataset, tokenizer, max_seq_length):
+    if "text_b" in dataset:
+        return tokenizer(
+            text=dataset["text_a"],
+            text_pair=dataset["text_b"],
+            truncation=True,
+            padding="max_length",
+            max_length=max_seq_length,
+        )
     else:
-        for example in examples:
-            text.append(example.text_a)
-            labels.append(example.label)
+        return tokenizer(
+            text=dataset["text"],
+            truncation=True,
+            padding="max_length",
+            max_length=max_seq_length,
+        )
 
-    examples = tokenizer.batch_encode_plus(
-        batch_text_or_text_pairs=text,
-        max_length=max_seq_length,
+
+def preprocess_data(text_a, text_b, labels, tokenizer, max_seq_length):
+    return tokenizer(
+        text=text_a,
+        text_pair=text_b,
         truncation=True,
         padding="max_length",
+        max_length=max_seq_length,
         return_tensors="pt",
     )
 
-    return [examples, labels]
-
     # if examples[0].text_b:
     #     tokenized_example = tokenizer.batch_encode_plus(
     #         text=example.text_a,
@@ -155,10 +158,18 @@ def preprocess_data(examples, tokenizer, max_seq_length):
     # return [tokenized_example, [example.label for example in data]]
 
 
-def build_classification_dataset(data, tokenizer, args, mode, multi_label, output_mode):
+def build_classification_dataset(
+    data, tokenizer, args, mode, multi_label, output_mode, no_cache
+):
     cached_features_file = os.path.join(
         args.cache_dir,
-        "cached_{}_{}_{}_{}_{}".format(mode, args.model_type, args.max_seq_length, len(args.labels_list), len(data),),
+        "cached_{}_{}_{}_{}_{}".format(
+            mode,
+            args.model_type,
+            args.max_seq_length,
+            len(args.labels_list),
+            len(data),
+        ),
     )
 
     if os.path.exists(cached_features_file) and (
@@ -167,16 +178,23 @@ def build_classification_dataset(data, tokenizer, args, mode, multi_label, outpu
     ):
         data = torch.load(cached_features_file)
         logger.info(f" Features loaded from cache at {cached_features_file}")
+        examples, labels = data
     else:
         logger.info(" Converting to features started. Cache is not used.")
 
+        if len(data) == 3:
+            # Sentence pair task
+            text_a, text_b, labels = data
+        else:
+            text_a, labels = data
+            text_b = None
+
         # If labels_map is defined, then labels need to be replaced with ints
         if args.labels_map and not args.regression:
-            for example in data:
-                if multi_label:
-                    example.label = [args.labels_map[label] for label in example.label]
-                else:
-                    example.label = args.labels_map[example.label]
+            if multi_label:
+                labels = [[args.labels_map[l] for l in label] for label in labels]
+            else:
+                labels = [args.labels_map[label] for label in labels]
 
         if (mode == "train" and args.use_multiprocessing) or (
             mode == "dev" and args.use_multiprocessing_for_evaluation
@@ -185,15 +203,40 @@ def build_classification_dataset(data, tokenizer, args, mode, multi_label, outpu
                 chunksize = max(len(data) // (args.process_count * 2), 500)
             else:
                 chunksize = args.multiprocessing_chunksize
-            data = [(data[i : i + chunksize], tokenizer, args.max_seq_length) for i in range(0, len(data), chunksize)]
+
+            if text_b is not None:
+                data = [
+                    (
+                        text_a[i : i + chunksize],
+                        text_b[i : i + chunksize],
+                        tokenizer,
+                        args.max_seq_length,
+                    )
+                    for i in range(0, len(text_a), chunksize)
+                ]
+            else:
+                data = [
+                    (text_a[i : i + chunksize], None, tokenizer, args.max_seq_length)
+                    for i in range(0, len(text_a), chunksize)
+                ]
+
             with Pool(args.process_count) as p:
-                examples, labels = zip(
-                    *tqdm(p.imap(preprocess_data_multiprocessing, data), total=len(data), disable=args.silent,)
+                examples = list(
+                    tqdm(
+                        p.imap(preprocess_data_multiprocessing, data),
+                        total=len(text_a),
+                        disable=args.silent,
+                    )
                 )
-            examples = {key: torch.cat([example[key] for example in examples]) for key in examples[0]}
-            labels = [label for label_sublist in labels for label in label_sublist]
+
+            examples = {
+                key: torch.cat([example[key] for example in examples])
+                for key in examples[0]
+            }
         else:
-            examples, labels = preprocess_data(data, tokenizer, args.max_seq_length)
+            examples = preprocess_data(
+                text_a, text_b, labels, tokenizer, args.max_seq_length
+            )
 
         if output_mode == "classification":
             labels = torch.tensor(labels, dtype=torch.long)
@@ -202,24 +245,74 @@ def build_classification_dataset(data, tokenizer, args, mode, multi_label, outpu
 
         data = (examples, labels)
 
-        if not args.no_cache:
+        if not args.no_cache and not no_cache:
             logger.info(" Saving features into cached file %s", cached_features_file)
             torch.save(data, cached_features_file)
 
-        return (examples, labels)
+    return (examples, labels)
 
 
 class ClassificationDataset(Dataset):
-    def __init__(self, data, tokenizer, args, mode, multi_label, output_mode):
+    def __init__(self, data, tokenizer, args, mode, multi_label, output_mode, no_cache):
         self.examples, self.labels = build_classification_dataset(
-            data, tokenizer, args, mode, multi_label, output_mode
+            data, tokenizer, args, mode, multi_label, output_mode, no_cache
         )
 
     def __len__(self):
         return len(self.examples["input_ids"])
 
     def __getitem__(self, index):
-        return {key: self.examples[key][index] for key in self.examples}, self.labels[index]
+        return (
+            {key: self.examples[key][index] for key in self.examples},
+            self.labels[index],
+        )
+
+
+def map_labels_to_numeric(example, multi_label, args):
+    if multi_label:
+        example["labels"] = [args.labels_map[label] for label in example["labels"]]
+    else:
+        example["labels"] = args.labels_map[example["labels"]]
+
+    return example
+
+
+def load_hf_dataset(data, tokenizer, args, multi_label):
+    if isinstance(data, str):
+        dataset = load_dataset(
+            "csv",
+            data_files=data,
+            delimiter="\t",
+            download_mode="force_redownload"
+            if args.reprocess_input_data
+            else "reuse_dataset_if_exists",
+        )
+    else:
+        dataset = HFDataset.from_pandas(data)
+
+    if args.labels_map and not args.regression:
+        dataset = dataset.map(lambda x: map_labels_to_numeric(x, multi_label, args))
+
+    dataset = dataset.map(
+        lambda x: preprocess_batch_for_hf_dataset(
+            x, tokenizer=tokenizer, max_seq_length=args.max_seq_length
+        ),
+        batched=True,
+    )
+
+    if args.model_type in ["bert", "xlnet", "albert", "layoutlm"]:
+        dataset.set_format(
+            type="pt",
+            columns=["input_ids", "token_type_ids", "attention_mask", "labels"],
+        )
+    else:
+        dataset.set_format(type="pt", columns=["input_ids", "attention_mask", "labels"])
+
+    if isinstance(data, str):
+        # This is not necessarily a train dataset. The datasets library insists on calling it train.
+        return dataset["train"]
+    else:
+        return dataset
 
 
 def convert_example_to_feature(
@@ -341,11 +434,15 @@ def convert_example_to_feature(
         padding_length = max_seq_length - len(input_ids)
         if pad_on_left:
             input_ids = ([pad_token] * padding_length) + input_ids
-            input_mask = ([0 if mask_padding_with_zero else 1] * padding_length) + input_mask
+            input_mask = (
+                [0 if mask_padding_with_zero else 1] * padding_length
+            ) + input_mask
             segment_ids = ([pad_token_segment_id] * padding_length) + segment_ids
         else:
             input_ids = input_ids + ([pad_token] * padding_length)
-            input_mask = input_mask + ([0 if mask_padding_with_zero else 1] * padding_length)
+            input_mask = input_mask + (
+                [0 if mask_padding_with_zero else 1] * padding_length
+            )
             segment_ids = segment_ids + ([pad_token_segment_id] * padding_length)
             if bboxes:
                 bboxes += [pad_token_box] * padding_length
@@ -367,11 +464,18 @@ def convert_example_to_feature(
 
     if bboxes:
         return InputFeatures(
-            input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_id=example.label, bboxes=bboxes
+            input_ids=input_ids,
+            input_mask=input_mask,
+            segment_ids=segment_ids,
+            label_id=example.label,
+            bboxes=bboxes,
         )
     else:
         return InputFeatures(
-            input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_id=example.label,
+            input_ids=input_ids,
+            input_mask=input_mask,
+            segment_ids=segment_ids,
+            label_id=example.label,
         )
 
 
@@ -416,12 +520,16 @@ def convert_example_to_feature_sliding_window(
         tokens_a = tokenizer.tokenize(example.text_a)
 
     if len(tokens_a) > bucket_size:
-        token_sets = [tokens_a[i : i + bucket_size] for i in range(0, len(tokens_a), stride)]
+        token_sets = [
+            tokens_a[i : i + bucket_size] for i in range(0, len(tokens_a), stride)
+        ]
     else:
         token_sets.append(tokens_a)
 
     if example.text_b:
-        raise ValueError("Sequence pair tasks not implemented for sliding window tokenization.")
+        raise ValueError(
+            "Sequence pair tasks not implemented for sliding window tokenization."
+        )
 
     # The convention in BERT is:
     # (a) For sequence pairs:
@@ -464,11 +572,15 @@ def convert_example_to_feature_sliding_window(
         padding_length = max_seq_length - len(input_ids)
         if pad_on_left:
             input_ids = ([pad_token] * padding_length) + input_ids
-            input_mask = ([0 if mask_padding_with_zero else 1] * padding_length) + input_mask
+            input_mask = (
+                [0 if mask_padding_with_zero else 1] * padding_length
+            ) + input_mask
             segment_ids = ([pad_token_segment_id] * padding_length) + segment_ids
         else:
             input_ids = input_ids + ([pad_token] * padding_length)
-            input_mask = input_mask + ([0 if mask_padding_with_zero else 1] * padding_length)
+            input_mask = input_mask + (
+                [0 if mask_padding_with_zero else 1] * padding_length
+            )
             segment_ids = segment_ids + ([pad_token_segment_id] * padding_length)
 
         assert len(input_ids) == max_seq_length
@@ -483,7 +595,12 @@ def convert_example_to_feature_sliding_window(
         #     raise KeyError(output_mode)
 
         input_features.append(
-            InputFeatures(input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_id=example.label,)
+            InputFeatures(
+                input_ids=input_ids,
+                input_mask=input_mask,
+                segment_ids=segment_ids,
+                label_id=example.label,
+            )
         )
 
     return input_features
@@ -516,11 +633,11 @@ def convert_examples_to_features(
     pad_to_max_length=True,
     args=None,
 ):
-    """ Loads a data file into a list of `InputBatch`s
-        `cls_token_at_end` define the location of the CLS token:
-            - False (Default, BERT/XLM pattern): [CLS] + A + [SEP] + B + [SEP]
-            - True (XLNet/GPT pattern): A + [SEP] + B + [SEP] + [CLS]
-        `cls_token_segment_id` define the segment id associated to the CLS token (0 for BERT, 2 for XLNet)
+    """Loads a data file into a list of `InputBatch`s
+    `cls_token_at_end` define the location of the CLS token:
+        - False (Default, BERT/XLM pattern): [CLS] + A + [SEP] + B + [SEP]
+        - True (XLNet/GPT pattern): A + [SEP] + B + [SEP] + [CLS]
+    `cls_token_segment_id` define the segment id associated to the CLS token (0 for BERT, 2 for XLNet)
     """
 
     examples = [
@@ -554,18 +671,26 @@ def convert_examples_to_features(
             with Pool(process_count) as p:
                 features = list(
                     tqdm(
-                        p.imap(convert_example_to_feature_sliding_window, examples, chunksize=chunksize,),
+                        p.imap(
+                            convert_example_to_feature_sliding_window,
+                            examples,
+                            chunksize=chunksize,
+                        ),
                         total=len(examples),
                         disable=silent,
                     )
                 )
             if flatten:
-                features = [feature for feature_set in features for feature in feature_set]
+                features = [
+                    feature for feature_set in features for feature in feature_set
+                ]
         else:
             with Pool(process_count) as p:
                 features = list(
                     tqdm(
-                        p.imap(convert_example_to_feature, examples, chunksize=chunksize),
+                        p.imap(
+                            convert_example_to_feature, examples, chunksize=chunksize
+                        ),
                         total=len(examples),
                         disable=silent,
                     )
@@ -573,12 +698,18 @@ def convert_examples_to_features(
     else:
         if sliding_window:
             features = [
-                convert_example_to_feature_sliding_window(example) for example in tqdm(examples, disable=silent)
+                convert_example_to_feature_sliding_window(example)
+                for example in tqdm(examples, disable=silent)
             ]
             if flatten:
-                features = [feature for feature_set in features for feature in feature_set]
+                features = [
+                    feature for feature_set in features for feature in feature_set
+                ]
         else:
-            features = [convert_example_to_feature(example) for example in tqdm(examples, disable=silent)]
+            features = [
+                convert_example_to_feature(example)
+                for example in tqdm(examples, disable=silent)
+            ]
 
     return features
 
@@ -601,7 +732,17 @@ def _truncate_seq_pair(tokens_a, tokens_b, max_length):
             tokens_b.pop()
 
 
-POOLING_BREAKDOWN = {1: (1, 1), 2: (2, 1), 3: (3, 1), 4: (2, 2), 5: (5, 1), 6: (3, 2), 7: (7, 1), 8: (4, 2), 9: (3, 3)}
+POOLING_BREAKDOWN = {
+    1: (1, 1),
+    2: (2, 1),
+    3: (3, 1),
+    4: (2, 2),
+    5: (5, 1),
+    6: (3, 2),
+    7: (7, 1),
+    8: (4, 2),
+    9: (3, 3),
+}
 
 
 class ImageEncoder(nn.Module):
@@ -649,17 +790,25 @@ class JsonlDataset(Dataset):
             files_list = json.load(open(files_list))
         if isinstance(data_path, str):
             if not files_list:
-                files_list = [f for f in os.listdir(data_path) if f.endswith(self.data_type_extension)]
+                files_list = [
+                    f
+                    for f in os.listdir(data_path)
+                    if f.endswith(self.data_type_extension)
+                ]
             self.data = [
                 dict(
-                    json.load(open(os.path.join(data_path, l + self.data_type_extension))),
+                    json.load(
+                        open(os.path.join(data_path, l + self.data_type_extension))
+                    ),
                     **{"images": l + image_type_extension},
                 )
                 for l in files_list
             ]
             self.data_dir = os.path.dirname(data_path)
         else:
-            data_path[self.images_label] = data_path[self.images_label].apply(lambda x: x + self.image_type_extension)
+            data_path[self.images_label] = data_path[self.images_label].apply(
+                lambda x: x + self.image_type_extension
+            )
             self.data = data_path.to_dict("records")
             self.data_dir = image_path
         self.tokenizer = tokenizer
@@ -673,17 +822,25 @@ class JsonlDataset(Dataset):
         return len(self.data)
 
     def __getitem__(self, index):
-        sentence = torch.LongTensor(self.tokenizer.encode(self.data[index][self.text_label], add_special_tokens=True))
+        sentence = torch.LongTensor(
+            self.tokenizer.encode(
+                self.data[index][self.text_label], add_special_tokens=True
+            )
+        )
         start_token, sentence, end_token = sentence[0], sentence[1:-1], sentence[-1]
         sentence = sentence[: self.max_seq_length]
 
         if self.multi_label:
             label = torch.zeros(self.n_classes)
-            label[[self.labels.index(tgt) for tgt in self.data[index][self.labels_label]]] = 1
+            label[
+                [self.labels.index(tgt) for tgt in self.data[index][self.labels_label]]
+            ] = 1
         else:
             label = torch.tensor(self.labels.index(self.data[index][self.labels_label]))
 
-        image = Image.open(os.path.join(self.data_dir, self.data[index]["images"])).convert("RGB")
+        image = Image.open(
+            os.path.join(self.data_dir, self.data[index]["images"])
+        ).convert("RGB")
         image = self.transforms(image)
 
         return {
@@ -717,7 +874,14 @@ def collate_fn(batch):
     img_start_token = torch.stack([row["image_start_token"] for row in batch])
     img_end_token = torch.stack([row["image_end_token"] for row in batch])
 
-    return text_tensor, mask_tensor, img_tensor, img_start_token, img_end_token, tgt_tensor
+    return (
+        text_tensor,
+        mask_tensor,
+        img_tensor,
+        img_start_token,
+        img_end_token,
+        tgt_tensor,
+    )
 
 
 def get_image_transforms():
@@ -726,7 +890,10 @@ def get_image_transforms():
             transforms.Resize(256),
             transforms.CenterCrop(224),
             transforms.ToTensor(),
-            transforms.Normalize(mean=[0.46777044, 0.44531429, 0.40661017], std=[0.12221994, 0.12145835, 0.14380469],),
+            transforms.Normalize(
+                mean=[0.46777044, 0.44531429, 0.40661017],
+                std=[0.12221994, 0.12145835, 0.14380469],
+            ),
         ]
     )
 
@@ -758,7 +925,11 @@ class LazyClassificationDataset(Dataset):
         return line_idx - start_row
 
     def __getitem__(self, idx):
-        line = linecache.getline(self.data_file, idx + 1 + self.start_row).rstrip("\n").split(self.delimiter)
+        line = (
+            linecache.getline(self.data_file, idx + 1 + self.start_row)
+            .rstrip("\n")
+            .split(self.delimiter)
+        )
 
         if not self.text_a_column and not self.text_b_column:
             text = line[self.text_column]
@@ -803,3 +974,18 @@ class LazyClassificationDataset(Dataset):
 
     def __len__(self):
         return self.num_entries
+
+
+def flatten_results(results, parent_key="", sep="/"):
+    out = []
+    if isinstance(results, collections.Mapping):
+        for key, value in results.items():
+            pkey = parent_key + sep + str(key) if parent_key else str(key)
+            out.extend(flatten_results(value, parent_key=pkey).items())
+    elif isinstance(results, collections.Iterable):
+        for key, value in enumerate(results):
+            pkey = parent_key + sep + str(key) if parent_key else str(key)
+            out.extend(flatten_results(value, parent_key=pkey).items())
+    else:
+        out.append((parent_key, results))
+    return dict(out)
