@@ -573,6 +573,7 @@ class MultiModalClassificationModel:
             )
             wandb.run._label(repo="simpletransformers")
             wandb.watch(self.model)
+            self.wandb_run_id = wandb.run.id
 
         if args.fp16:
             from torch.cuda import amp
@@ -1035,7 +1036,13 @@ class MultiModalClassificationModel:
         return result, model_outputs
 
     def evaluate(
-        self, eval_dataset, output_dir, prefix="", verbose=True, silent=False, **kwargs,
+        self,
+        eval_dataset,
+        output_dir,
+        prefix="",
+        verbose=True,
+        silent=False,
+        **kwargs,
     ):
         """
         Evaluates the model on eval_df.
