@@ -15,27 +15,46 @@ from transformers import (
     XLNetModel,
     XLNetPreTrainedModel,
 )
+from transformers.modeling_utils import PreTrainedModel, SequenceSummary
+from transformers.models.albert.modeling_albert import (
+    AlbertConfig,
+    AlbertModel,
+    AlbertPreTrainedModel,
+)
+from transformers.models.big_bird.configuration_big_bird import BigBirdConfig
+from transformers.models.big_bird.modeling_big_bird import (
+    BigBirdClassificationHead,
+    BigBirdModel,
+    BigBirdPreTrainedModel,
+)
 from transformers.models.camembert.configuration_camembert import CamembertConfig
+from transformers.models.camembert.modeling_camembert import (
+    CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST,
+)
 from transformers.models.distilbert.configuration_distilbert import DistilBertConfig
-from transformers.models.roberta.configuration_roberta import RobertaConfig
-from transformers.models.xlm_roberta.configuration_xlm_roberta import XLMRobertaConfig
-from transformers.models.albert.modeling_albert import AlbertConfig, AlbertModel, AlbertPreTrainedModel
-from transformers.models.distilbert.modeling_distilbert import DISTILBERT_PRETRAINED_MODEL_ARCHIVE_LIST
+from transformers.models.distilbert.modeling_distilbert import (
+    DISTILBERT_PRETRAINED_MODEL_ARCHIVE_LIST,
+)
 from transformers.models.electra.modeling_electra import (
     ELECTRA_PRETRAINED_MODEL_ARCHIVE_LIST,
     ElectraConfig,
     ElectraModel,
     ElectraPreTrainedModel,
 )
-from transformers.models.camembert.modeling_camembert import CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST
+from transformers.models.longformer.modeling_longformer import (
+    LongformerClassificationHead,
+    LongformerPreTrainedModel,
+)
+from transformers.models.roberta.configuration_roberta import RobertaConfig
 from transformers.models.roberta.modeling_roberta import (
     ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST,
     RobertaClassificationHead,
     RobertaForQuestionAnswering,
 )
-from transformers.modeling_utils import PreTrainedModel, SequenceSummary
-from transformers.models.xlm_roberta.modeling_xlm_roberta import XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
-from transformers.models.longformer.modeling_longformer import LongformerClassificationHead, LongformerPreTrainedModel
+from transformers.models.xlm_roberta.configuration_xlm_roberta import XLMRobertaConfig
+from transformers.models.xlm_roberta.modeling_xlm_roberta import (
+    XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST,
+)
 
 
 class BertForMultiLabelSequenceClassification(BertPreTrainedModel):
@@ -54,7 +73,13 @@ class BertForMultiLabelSequenceClassification(BertPreTrainedModel):
         self.init_weights()
 
     def forward(
-        self, input_ids, attention_mask=None, token_type_ids=None, position_ids=None, head_mask=None, labels=None,
+        self,
+        input_ids,
+        attention_mask=None,
+        token_type_ids=None,
+        position_ids=None,
+        head_mask=None,
+        labels=None,
     ):
         outputs = self.bert(
             input_ids,
@@ -69,12 +94,16 @@ class BertForMultiLabelSequenceClassification(BertPreTrainedModel):
         pooled_output = self.dropout(pooled_output)
         logits = self.classifier(pooled_output)
 
-        outputs = (logits,) + outputs[2:]  # add hidden states and attention if they are here
+        outputs = (logits,) + outputs[
+            2:
+        ]  # add hidden states and attention if they are here
 
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs  # (loss), logits, (hidden_states), (attentions)
@@ -121,13 +150,17 @@ class RobertaForMultiLabelSequenceClassification(BertPreTrainedModel):
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
 
 
-class BertweetForMultiLabelSequenceClassification(RobertaForMultiLabelSequenceClassification):
+class BertweetForMultiLabelSequenceClassification(
+    RobertaForMultiLabelSequenceClassification
+):
     """
     BERTweet model adapted for multi-label sequence classification.
     BERTweet shares the Roberta architecture, so we can reuse the simpletransformers
@@ -137,7 +170,9 @@ class BertweetForMultiLabelSequenceClassification(RobertaForMultiLabelSequenceCl
     base_model_prefix = "bertweet"
 
 
-class CamembertForMultiLabelSequenceClassification(RobertaForMultiLabelSequenceClassification):
+class CamembertForMultiLabelSequenceClassification(
+    RobertaForMultiLabelSequenceClassification
+):
     """
     Camembert model adapted for multi-label sequence classification.
     Camembert shares the Roberta architecture, so we can reuse the simpletransformers
@@ -195,12 +230,16 @@ class XLNetForMultiLabelSequenceClassification(XLNetPreTrainedModel):
         output = self.sequence_summary(output)
         logits = self.logits_proj(output)
 
-        outputs = (logits,) + transformer_outputs[1:]  # Keep mems, hidden states, attentions if there are in it
+        outputs = (logits,) + transformer_outputs[
+            1:
+        ]  # Keep mems, hidden states, attentions if there are in it
 
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
@@ -248,20 +287,24 @@ class XLMForMultiLabelSequenceClassification(XLMPreTrainedModel):
         output = transformer_outputs[0]
         logits = self.sequence_summary(output)
 
-        outputs = (logits,) + transformer_outputs[1:]  # Keep new_mems and attention/hidden states if they are here
+        outputs = (logits,) + transformer_outputs[
+            1:
+        ]  # Keep new_mems and attention/hidden states if they are here
 
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
 
 
 class DistilBertPreTrainedModel(PreTrainedModel):
-    """ An abstract class to handle weights initialization and
-        a simple interface for downloading and loading pretrained models.
+    """An abstract class to handle weights initialization and
+    a simple interface for downloading and loading pretrained models.
     """
 
     config_class = DistilBertConfig
@@ -270,8 +313,7 @@ class DistilBertPreTrainedModel(PreTrainedModel):
     base_model_prefix = "distilbert"
 
     def _init_weights(self, module):
-        """ Initialize the weights.
-        """
+        """Initialize the weights."""
         if isinstance(module, nn.Embedding):
             if module.weight.requires_grad:
                 module.weight.data.normal_(mean=0.0, std=self.config.initializer_range)
@@ -302,9 +344,16 @@ class DistilBertForMultiLabelSequenceClassification(DistilBertPreTrainedModel):
         self.init_weights()
 
     def forward(
-        self, input_ids=None, attention_mask=None, head_mask=None, inputs_embeds=None, labels=None,
+        self,
+        input_ids=None,
+        attention_mask=None,
+        head_mask=None,
+        inputs_embeds=None,
+        labels=None,
     ):
-        distilbert_output = self.distilbert(input_ids=input_ids, attention_mask=attention_mask, head_mask=head_mask)
+        distilbert_output = self.distilbert(
+            input_ids=input_ids, attention_mask=attention_mask, head_mask=head_mask
+        )
         hidden_state = distilbert_output[0]  # (bs, seq_len, dim)
         pooled_output = hidden_state[:, 0]  # (bs, dim)
         pooled_output = self.pre_classifier(pooled_output)  # (bs, dim)
@@ -316,7 +365,9 @@ class DistilBertForMultiLabelSequenceClassification(DistilBertPreTrainedModel):
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
@@ -364,12 +415,16 @@ class AlbertForMultiLabelSequenceClassification(AlbertPreTrainedModel):
         pooled_output = self.dropout(pooled_output)
         logits = self.classifier(pooled_output)
 
-        outputs = (logits,) + outputs[2:]  # add hidden states and attention if they are here
+        outputs = (logits,) + outputs[
+            2:
+        ]  # add hidden states and attention if they are here
 
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs  # (loss), logits, (hidden_states), (attentions)
@@ -417,12 +472,16 @@ class FlaubertForMultiLabelSequenceClassification(FlaubertModel):
         output = transformer_outputs[0]
         logits = self.sequence_summary(output)
 
-        outputs = (logits,) + transformer_outputs[1:]  # Keep new_mems and attention/hidden states if they are here
+        outputs = (logits,) + transformer_outputs[
+            1:
+        ]  # Keep new_mems and attention/hidden states if they are here
 
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
@@ -472,13 +531,17 @@ class LongformerForMultiLabelSequenceClassification(LongformerPreTrainedModel):
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
 
 
-class XLMRobertaForMultiLabelSequenceClassification(RobertaForMultiLabelSequenceClassification):
+class XLMRobertaForMultiLabelSequenceClassification(
+    RobertaForMultiLabelSequenceClassification
+):
     config_class = XLMRobertaConfig
     pretrained_model_archive_map = XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
 
@@ -516,14 +579,19 @@ class ElectraForLanguageModelingModel(PreTrainedModel):
             self.tie_generator_and_discriminator_embeddings()
 
     def tie_generator_and_discriminator_embeddings(self):
-        self.discriminator_model.set_input_embeddings(self.generator_model.get_input_embeddings())
+        self.discriminator_model.set_input_embeddings(
+            self.generator_model.get_input_embeddings()
+        )
 
     def forward(self, inputs, labels, attention_mask=None, token_type_ids=None):
         d_inputs = inputs.clone()
 
         # run masked LM.
         g_out = self.generator_model(
-            inputs, labels=labels, attention_mask=attention_mask, token_type_ids=token_type_ids
+            inputs,
+            labels=labels,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
         )
 
         # get samples from masked LM.
@@ -547,7 +615,10 @@ class ElectraForLanguageModelingModel(PreTrainedModel):
 
         # run token classification, predict whether each token was corrupted.
         d_out = self.discriminator_model(
-            d_inputs, labels=d_labels, attention_mask=attention_mask, token_type_ids=token_type_ids
+            d_inputs,
+            labels=d_labels,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
         )
 
         g_loss = g_out[0]
@@ -605,7 +676,14 @@ class ElectraForSequenceClassification(ElectraPreTrainedModel):
         labels=None,
     ):
 
-        outputs = self.electra(input_ids, attention_mask, token_type_ids, position_ids, head_mask, inputs_embeds)
+        outputs = self.electra(
+            input_ids,
+            attention_mask,
+            token_type_ids,
+            position_ids,
+            head_mask,
+            inputs_embeds,
+        )
         sequence_output = outputs[0]
         pooled_output = self.pooler(sequence_output)
         pooled_output = self.dropout(pooled_output)
@@ -655,7 +733,14 @@ class ElectraForMultiLabelSequenceClassification(ElectraPreTrainedModel):
         labels=None,
     ):
 
-        outputs = self.electra(input_ids, attention_mask, token_type_ids, position_ids, head_mask, inputs_embeds)
+        outputs = self.electra(
+            input_ids,
+            attention_mask,
+            token_type_ids,
+            position_ids,
+            head_mask,
+            inputs_embeds,
+        )
         sequence_output = outputs[0]
         pooled_output = self.pooler(sequence_output)
         pooled_output = self.dropout(pooled_output)
@@ -665,7 +750,9 @@ class ElectraForMultiLabelSequenceClassification(ElectraPreTrainedModel):
         if labels is not None:
             loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
             labels = labels.float()
-            loss = loss_fct(logits.view(-1, self.num_labels), labels.view(-1, self.num_labels))
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
             outputs = (loss,) + outputs
 
         return outputs
@@ -701,7 +788,14 @@ class ElectraForQuestionAnswering(ElectraPreTrainedModel):
         end_positions=None,
     ):
 
-        outputs = self.electra(input_ids, attention_mask, token_type_ids, position_ids, head_mask, inputs_embeds)
+        outputs = self.electra(
+            input_ids,
+            attention_mask,
+            token_type_ids,
+            position_ids,
+            head_mask,
+            inputs_embeds,
+        )
         sequence_output = outputs[0]
         logits = self.qa_outputs(sequence_output)
 
@@ -709,7 +803,10 @@ class ElectraForQuestionAnswering(ElectraPreTrainedModel):
         start_logits = start_logits.squeeze(-1)
         end_logits = end_logits.squeeze(-1)
 
-        outputs = (start_logits, end_logits,) + outputs[2:]
+        outputs = (
+            start_logits,
+            end_logits,
+        ) + outputs[2:]
         if start_positions is not None and end_positions is not None:
             # If we are on multi-GPU, split add a dimension
             if len(start_positions.size()) > 1:
@@ -727,9 +824,61 @@ class ElectraForQuestionAnswering(ElectraPreTrainedModel):
             total_loss = (start_loss + end_loss) / 2
             outputs = (total_loss,) + outputs
 
-        return outputs  # (loss), start_logits, end_logits, (hidden_states), (attentions)
+        return (
+            outputs  # (loss), start_logits, end_logits, (hidden_states), (attentions)
+        )
 
 
 class XLMRobertaForQuestionAnswering(RobertaForQuestionAnswering):
     config_class = XLMRobertaConfig
     pretrained_model_archive_map = XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
+
+
+class BigBirdForMultiLabelSequenceClassification(BigBirdPreTrainedModel):
+    """
+    Bert model adapted for multi-label sequence classification
+    """
+
+    def __init__(self, config, pos_weight=None):
+        super(BigBirdForMultiLabelSequenceClassification, self).__init__(config)
+        self.num_labels = config.num_labels
+        self.config = config
+        self.bert = BigBirdModel(config)
+        self.classifier = BigBirdClassificationHead(config)
+        self.pos_weight = pos_weight
+
+        self.init_weights()
+
+    def forward(
+        self,
+        input_ids,
+        attention_mask=None,
+        token_type_ids=None,
+        position_ids=None,
+        head_mask=None,
+        labels=None,
+    ):
+        outputs = self.bert(
+            input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            head_mask=head_mask,
+        )
+
+        sequence_output = outputs[0]
+        logits = self.classifier(sequence_output)
+
+        outputs = (logits,) + outputs[
+            2:
+        ]  # add hidden states and attention if they are here
+
+        if labels is not None:
+            loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
+            labels = labels.float()
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
+            outputs = (loss,) + outputs
+
+        return outputs  # (loss), logits, (hidden_states), (attentions)
