@@ -16,7 +16,8 @@
 """ Named entity recognition fine-tuning: utilities to work with CoNLL-2003 task. """
 
 from __future__ import absolute_import, division, print_function
-
+import enum
+import collections
 import linecache
 import logging
 import os
@@ -29,26 +30,40 @@ from torch.functional import split
 from torch.nn import CrossEntropyLoss
 from torch.utils.data import Dataset
 from tqdm.auto import tqdm
+from datasets import load_dataset
+from datasets import Dataset as HFDataset
 
 
 class InputExample(object):
     """A single training/test example for token classification."""
 
-    def __init__(self, guid, words, labels, x0=None, y0=None, x1=None, y1=None):
+    def __init__(
+        self,
+        guid,
+        words,
+        labels,
+        x0=None,
+        y0=None,
+        x1=None,
+        y1=None,
+        tokenized_word_ids=None,
+    ):
         """Constructs a InputExample.
         Args:
             guid: Unique id for the example.
             words: list. The words of the sequence.
-            labels: (Optional) list. The labels for each word of the sequence. This should be
+            labels: list. The labels for each word of the sequence. This should be
             specified for train and dev examples, but not for test examples.
             x0: (Optional) list. The list of x0 coordinates for each word.
             y0: (Optional) list. The list of y0 coordinates for each word.
             x1: (Optional) list. The list of x1 coordinates for each word.
             y1: (Optional) list. The list of y1 coordinates for each word.
+            tokenized_word_ids: (Optional) list. Tokenized words converted to input_ids
         """
         self.guid = guid
         self.words = words
         self.labels = labels
+        self.tokenized_word_ids = tokenized_word_ids
         if x0 is None:
             self.bboxes = None
         else:
@@ -97,14 +112,14 @@ def read_examples_from_file(data_file, mode, bbox=False):
                         x1 = []
                         y1 = []
                 else:
-                    splits = line.split(" ")
+                    splits = line.split()
                     words.append(splits[0])
                     if len(splits) > 1:
-                        labels.append(splits[1].replace("\n", ""))
-                        x0.append(split[2].replace("\n", ""))
-                        y0.append(split[3].replace("\n", ""))
-                        x1.append(split[4].replace("\n", ""))
-                        y1.append(split[5].replace("\n", ""))
+                        labels.append(splits[1])
+                        x0.append(splits[2])
+                        y0.append(splits[3])
+                        x1.append(splits[4])
+                        y1.append(splits[5])
                     else:
                         # Examples could have no label for mode = "test"
                         labels.append("O")
@@ -112,7 +127,11 @@ def read_examples_from_file(data_file, mode, bbox=False):
                 if line.startswith("-DOCSTART-") or line == "" or line == "\n":
                     if words:
                         examples.append(
-                            InputExample(guid="{}-{}".format(mode, guid_index), words=words, labels=labels)
+                            InputExample(
+                                guid="{}-{}".format(mode, guid_index),
+                                words=words,
+                                labels=labels,
+                            )
                         )
                         guid_index += 1
                         words = []
@@ -129,11 +148,23 @@ def read_examples_from_file(data_file, mode, bbox=False):
             if bbox:
                 examples.append(
                     InputExample(
-                        guid="%s-%d".format(mode, guid_index), words=words, labels=labels, x0=x0, y0=y0, x1=x1, y1=y1
+                        guid="%s-%d".format(mode, guid_index),
+                        words=words,
+                        labels=labels,
+                        x0=x0,
+                        y0=y0,
+                        x1=x1,
+                        y1=y1,
                     )
                 )
             else:
-                examples.append(InputExample(guid="%s-%d".format(mode, guid_index), words=words, labels=labels))
+                examples.append(
+                    InputExample(
+                        guid="%s-%d".format(mode, guid_index),
+                        words=words,
+                        labels=labels,
+                    )
+                )
     return examples
 
 
@@ -153,7 +184,11 @@ def get_examples_from_df(data, bbox=False):
         ]
     else:
         return [
-            InputExample(guid=sentence_id, words=sentence_df["words"].tolist(), labels=sentence_df["labels"].tolist(),)
+            InputExample(
+                guid=sentence_id,
+                words=sentence_df["words"].tolist(),
+                labels=sentence_df["labels"].tolist(),
+            )
             for sentence_id, sentence_df in data.groupby(["sentence_id"])
         ]
 
@@ -215,16 +250,24 @@ def convert_example_to_feature(
     pad_token_label_id,
     sequence_a_segment_id,
     mask_padding_with_zero,
+    return_input_feature=True,
 ):
     tokens = []
     label_ids = []
     bboxes = []
     if example.bboxes:
-        for word, label, bbox in zip(example.words, example.labels, example.bboxes):
-            word_tokens = tokenizer.tokenize(word)
+        for i, (word, label, bbox) in enumerate(
+            zip(example.words, example.labels, example.bboxes)
+        ):
+            if example.tokenized_word_ids is None:
+                word_tokens = tokenizer.tokenize(word)
+            else:
+                word_tokens = example.tokenized_word_ids[i]
             tokens.extend(word_tokens)
             # Use the real label id for the first token of the word, and padding ids for the remaining tokens
-            label_ids.extend([label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1))
+            label_ids.extend(
+                [label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1)
+            )
             bboxes.extend([bbox] * len(word_tokens))
 
         cls_token_box = [0, 0, 0, 0]
@@ -232,12 +275,19 @@ def convert_example_to_feature(
         pad_token_box = [0, 0, 0, 0]
 
     else:
-        for word, label in zip(example.words, example.labels):
-            word_tokens = tokenizer.tokenize(word)
+        for i, (word, label) in enumerate(zip(example.words, example.labels)):
+            if example.tokenized_word_ids is None:
+                word_tokens = tokenizer.tokenize(word)
+            else:
+                word_tokens = example.tokenized_word_ids[i]
             tokens.extend(word_tokens)
             # Use the real label id for the first token of the word, and padding ids for the remaining tokens
-            if word_tokens:  # avoid non printable character like '\u200e' which are tokenized as a void token ''
-                label_ids.extend([label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1))
+            if (
+                word_tokens
+            ):  # avoid non printable character like '\u200e' which are tokenized as a void token ''
+                label_ids.extend(
+                    [label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1)
+                )
 
     # Account for [CLS] and [SEP] with "- 2" and with "- 3" for RoBERTa.
     special_tokens_count = 3 if sep_token_extra else 2
@@ -288,7 +338,10 @@ def convert_example_to_feature(
         if bboxes:
             bboxes = [cls_token_box] + bboxes
 
-    input_ids = tokenizer.convert_tokens_to_ids(tokens)
+    if example.tokenized_word_ids is None:
+        input_ids = tokenizer.convert_tokens_to_ids(tokens)
+    else:
+        input_ids = tokens
 
     # The mask has 1 for real tokens and 0 for padding tokens. Only real
     # tokens are attended to.
@@ -298,7 +351,9 @@ def convert_example_to_feature(
     padding_length = max_seq_length - len(input_ids)
     if pad_on_left:
         input_ids = ([pad_token] * padding_length) + input_ids
-        input_mask = ([0 if mask_padding_with_zero else 1] * padding_length) + input_mask
+        input_mask = (
+            [0 if mask_padding_with_zero else 1] * padding_length
+        ) + input_mask
         segment_ids = ([pad_token_segment_id] * padding_length) + segment_ids
         label_ids = ([pad_token_label_id] * padding_length) + label_ids
     else:
@@ -316,12 +371,38 @@ def convert_example_to_feature(
     if bboxes:
         assert len(bboxes) == max_seq_length
 
-    if bboxes:
-        return InputFeatures(
-            input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_ids=label_ids, bboxes=bboxes
-        )
+    if return_input_feature:
+        if bboxes:
+            return InputFeatures(
+                input_ids=input_ids,
+                input_mask=input_mask,
+                segment_ids=segment_ids,
+                label_ids=label_ids,
+                bboxes=bboxes,
+            )
+        else:
+            return InputFeatures(
+                input_ids=input_ids,
+                input_mask=input_mask,
+                segment_ids=segment_ids,
+                label_ids=label_ids,
+            )
     else:
-        return InputFeatures(input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_ids=label_ids,)
+        if bboxes:
+            return (
+                input_ids,
+                input_mask,
+                segment_ids,
+                label_ids,
+                bboxes,
+            )
+        else:
+            return (
+                input_ids,
+                input_mask,
+                segment_ids,
+                label_ids,
+            )
 
 
 def convert_examples_to_features(
@@ -347,16 +428,18 @@ def convert_examples_to_features(
     mode="dev",
     use_multiprocessing_for_evaluation=False,
 ):
-    """ Loads a data file into a list of `InputBatch`s
-        `cls_token_at_end` define the location of the CLS token:
-            - False (Default, BERT/XLM pattern): [CLS] + A + [SEP] + B + [SEP]
-            - True (XLNet/GPT pattern): A + [SEP] + B + [SEP] + [CLS]
-        `cls_token_segment_id` define the segment id associated to the CLS token (0 for BERT, 2 for XLNet)
+    """Loads a data file into a list of `InputBatch`s
+    `cls_token_at_end` define the location of the CLS token:
+        - False (Default, BERT/XLM pattern): [CLS] + A + [SEP] + B + [SEP]
+        - True (XLNet/GPT pattern): A + [SEP] + B + [SEP] + [CLS]
+    `cls_token_segment_id` define the segment id associated to the CLS token (0 for BERT, 2 for XLNet)
     """
 
     label_map = {label: i for i, label in enumerate(label_list)}
 
-    if (mode == "train" and use_multiprocessing) or (mode == "dev" and use_multiprocessing_for_evaluation):
+    if (mode == "train" and use_multiprocessing) or (
+        mode == "dev" and use_multiprocessing_for_evaluation
+    ):
         if chunksize == -1:
             chunksize = max(len(examples) // (process_count * 2), 500)
         examples = [
@@ -383,13 +466,19 @@ def convert_examples_to_features(
         with Pool(process_count) as p:
             features = list(
                 tqdm(
-                    p.imap(convert_examples_with_multiprocessing, examples, chunksize=chunksize),
+                    p.imap(
+                        convert_examples_with_multiprocessing,
+                        examples,
+                        chunksize=chunksize,
+                    ),
                     total=len(examples),
                     disable=silent,
                 )
             )
 
-            features = [feature for feature_group in features for feature in feature_group]
+            features = [
+                feature for feature_group in features for feature in feature_group
+            ]
     else:
         features = [
             convert_example_to_feature(
@@ -435,11 +524,165 @@ def get_labels(path):
         ]
 
 
+def preprocess_batch_for_hf_dataset(
+    data,
+    label_list,
+    max_seq_length,
+    tokenizer,
+    cls_token_at_end=False,
+    cls_token="[CLS]",
+    cls_token_segment_id=1,
+    sep_token="[SEP]",
+    sep_token_extra=False,
+    pad_on_left=False,
+    pad_token=0,
+    pad_token_segment_id=0,
+    pad_token_label_id=-1,
+    sequence_a_segment_id=0,
+    mask_padding_with_zero=True,
+    silent=False,
+):
+    sequence_lengths = []
+    all_words = []
+    for seq in data["words"]:
+        sequence_lengths.append(len(seq))
+        all_words.extend(seq)  # Need to check whether adding the prefix space helps
+
+    tokenized_word_ids_all = tokenizer(text=all_words, add_special_tokens=False)[
+        "input_ids"
+    ]
+
+    tokenized_word_ids_batch = []
+    tokenized_word_ids_batch = [
+        tokenized_word_ids_all[
+            len(tokenized_word_ids_batch) : len(tokenized_word_ids_batch) + seq_len
+        ]
+        for seq_len in sequence_lengths
+    ]
+
+    examples = [
+        InputExample(guid, words, labels, tokenized_word_ids=tokenized_ids)
+        for guid, words, labels, tokenized_ids in zip(
+            data["sentence_id"], data["words"], data["labels"], tokenized_word_ids_batch
+        )
+    ]
+    label_map = {label: i for i, label in enumerate(label_list)}
+
+    features = [
+        convert_example_to_feature(
+            example,
+            label_map=label_map,
+            max_seq_length=max_seq_length,
+            tokenizer=tokenizer,
+            cls_token_at_end=cls_token_at_end,
+            cls_token=cls_token,
+            cls_token_segment_id=cls_token_segment_id,
+            sep_token=sep_token,
+            sep_token_extra=sep_token_extra,
+            pad_on_left=pad_on_left,
+            pad_token=pad_token,
+            pad_token_segment_id=pad_token_segment_id,
+            pad_token_label_id=pad_token_label_id,
+            sequence_a_segment_id=sequence_a_segment_id,
+            mask_padding_with_zero=mask_padding_with_zero,
+            return_input_feature=False,
+        )
+        for example in tqdm(examples, disable=silent)
+    ]
+
+    feature_dict = {}
+    feature_names = [
+        "input_ids",
+        "attention_mask",
+        "token_type_ids",
+        "labels",
+    ]
+
+    for i, feature in enumerate(zip(*features)):
+        feature_dict[feature_names[i]] = list(feature)
+
+    return feature_dict
+
+
+def load_hf_dataset(
+    data,
+    tokenizer,
+    label_list,
+    max_seq_length,
+    cls_token_at_end=False,
+    cls_token="[CLS]",
+    cls_token_segment_id=1,
+    sep_token="[SEP]",
+    sep_token_extra=False,
+    pad_on_left=False,
+    pad_token=0,
+    pad_token_segment_id=0,
+    pad_token_label_id=-1,
+    sequence_a_segment_id=0,
+    mask_padding_with_zero=True,
+    silent=False,
+    args=None,
+):
+    if isinstance(data, str):
+        # dataset = load_dataset("conll2003", data_files=data)
+        dataset = load_dataset(
+            os.path.join(
+                os.path.dirname(os.path.abspath(__file__)), "ner_dataset_loading_script"
+            ),
+            data_files=data,
+            download_mode="force_redownload"
+            if args.reprocess_input_data
+            else "reuse_dataset_if_exists",
+        )
+    else:
+        raise TypeError(
+            "{} is not a path to a data file (e.g. tsv). The input must be a data file for NERModel.".format(
+                data
+            )
+        )
+
+    dataset = dataset.map(
+        lambda x: preprocess_batch_for_hf_dataset(
+            x,
+            tokenizer,
+            label_list,
+            max_seq_length,
+            cls_token_at_end=cls_token_at_end,
+            cls_token=cls_token,
+            cls_token_segment_id=cls_token_segment_id,
+            sep_token=sep_token,
+            sep_token_extra=sep_token_extra,
+            pad_on_left=pad_on_left,
+            pad_token=pad_token,
+            pad_token_segment_id=pad_token_segment_id,
+            pad_token_label_id=pad_token_label_id,
+            sequence_a_segment_id=sequence_a_segment_id,
+            mask_padding_with_zero=mask_padding_with_zero,
+            silent=silent,
+        ),
+        batched=True,
+    )
+
+    dataset.set_format(
+        type="pt", columns=["input_ids", "token_type_ids", "attention_mask", "labels"]
+    )
+
+    if isinstance(data, str):
+        # This is not necessarily a train dataset. The datasets library insists on calling it train.
+        return dataset["train"]
+    else:
+        return dataset
+
+
 class LazyNERDataset(Dataset):
     def __init__(self, data_file, tokenizer, args):
         self.data_file = data_file
-        self.lazy_loading_start_line = args.lazy_loading_start_line if args.lazy_loading_start_line else 0
-        self.example_lines, self.num_entries = self._get_examples(self.data_file, self.lazy_loading_start_line)
+        self.lazy_loading_start_line = (
+            args.lazy_loading_start_line if args.lazy_loading_start_line else 0
+        )
+        self.example_lines, self.num_entries = self._get_examples(
+            self.data_file, self.lazy_loading_start_line
+        )
         self.tokenizer = tokenizer
         self.args = args
         self.pad_token_label_id = CrossEntropyLoss().ignore_index
@@ -471,7 +714,9 @@ class LazyNERDataset(Dataset):
                 # Examples could have no label for mode = "test"
                 labels.append("O")
         if words:
-            example = InputExample(guid="%s-%d".format("train", idx), words=words, labels=labels)
+            example = InputExample(
+                guid="%s-%d".format("train", idx), words=words, labels=labels
+            )
 
         label_map = {label: i for i, label in enumerate(self.args.labels_list)}
 
@@ -493,7 +738,7 @@ class LazyNERDataset(Dataset):
             True,
         )
 
-        features = convert_example_to_feature(example_row)
+        features = convert_example_to_feature(*example_row)
         all_input_ids = torch.tensor(features.input_ids, dtype=torch.long)
         all_input_mask = torch.tensor(features.input_mask, dtype=torch.long)
         all_segment_ids = torch.tensor(features.segment_ids, dtype=torch.long)
@@ -502,3 +747,18 @@ class LazyNERDataset(Dataset):
 
     def __len__(self):
         return self.num_entries
+
+
+def flatten_results(results, parent_key="", sep="/"):
+    out = []
+    if isinstance(results, collections.Mapping):
+        for key, value in results.items():
+            pkey = parent_key + sep + str(key) if parent_key else str(key)
+            out.extend(flatten_results(value, parent_key=pkey).items())
+    elif isinstance(results, collections.Iterable):
+        for key, value in enumerate(results):
+            pkey = parent_key + sep + str(key) if parent_key else str(key)
+            out.extend(flatten_results(value, parent_key=pkey).items())
+    else:
+        out.append((parent_key, results))
+    return dict(out)
