@@ -16,14 +16,30 @@ from pprint import pprint
 
 import torch
 from tensorboardX import SummaryWriter
-from torch.utils.data import DataLoader, Dataset, RandomSampler, SequentialSampler, TensorDataset
+from torch.utils.data import (
+    DataLoader,
+    Dataset,
+    RandomSampler,
+    SequentialSampler,
+    TensorDataset,
+)
 from tqdm import tqdm, trange
-from transformers import AdamW, SquadExample, XLMTokenizer, get_linear_schedule_with_warmup
+from transformers import (
+    AdamW,
+    SquadExample,
+    XLMTokenizer,
+    get_linear_schedule_with_warmup,
+)
 from transformers.data.processors.squad import (
     squad_convert_example_to_features,
     squad_convert_example_to_features_init,
 )
-from transformers.models.bert.tokenization_bert import BasicTokenizer, whitespace_tokenize
+from transformers.models.bert.tokenization_bert import (
+    BasicTokenizer,
+    whitespace_tokenize,
+)
+from datasets import load_dataset
+from datasets import Dataset as HFDataset
 
 logger = logging.getLogger(__name__)
 
@@ -206,7 +222,11 @@ def convert_example_to_feature(example_row):
         else:
             tok_end_position = len(all_doc_tokens) - 1
         (tok_start_position, tok_end_position) = _improve_answer_span(
-            all_doc_tokens, tok_start_position, tok_end_position, tokenizer, example.orig_answer_text,
+            all_doc_tokens,
+            tok_start_position,
+            tok_end_position,
+            tokenizer,
+            example.orig_answer_text,
         )
 
     # The -3 accounts for [CLS], [SEP] and [SEP]
@@ -215,7 +235,9 @@ def convert_example_to_feature(example_row):
     # We can have documents that are longer than the maximum sequence length.
     # To deal with this we do a sliding window approach, where we take chunks
     # of the up to our max length with a stride of `doc_stride`.
-    _DocSpan = collections.namedtuple("DocSpan", ["start", "length"])  # pylint: disable=invalid-name
+    _DocSpan = collections.namedtuple(
+        "DocSpan", ["start", "length"]
+    )  # pylint: disable=invalid-name
     doc_spans = []
     start_offset = 0
     while start_offset < len(all_doc_tokens):
@@ -262,7 +284,9 @@ def convert_example_to_feature(example_row):
             split_token_index = doc_span.start + i
             token_to_orig_map[len(tokens)] = tok_to_orig_index[split_token_index]
 
-            is_max_context = _check_is_max_context(doc_spans, doc_span_index, split_token_index)
+            is_max_context = _check_is_max_context(
+                doc_spans, doc_span_index, split_token_index
+            )
             token_is_max_context[len(tokens)] = is_max_context
             tokens.append(all_doc_tokens[split_token_index])
             if not sequence_a_is_doc:
@@ -433,12 +457,18 @@ def squad_convert_examples_to_features(
     # Defining helper methods
     features = []
     threads = min(threads, cpu_count())
-    if (is_training and args.use_multiprocessing) or (not is_training and args.use_multiprocessing_for_evaluation):
+    if (is_training and args.use_multiprocessing) or (
+        not is_training and args.use_multiprocessing_for_evaluation
+    ):
         if args.multiprocessing_chunksize == -1:
             chunksize = max(len(examples) // (args.process_count * 2), 500)
         else:
             chunksize = args.multiprocessing_chunksize
-        with Pool(threads, initializer=squad_convert_example_to_features_init, initargs=(tokenizer,)) as p:
+        with Pool(
+            threads,
+            initializer=squad_convert_example_to_features_init,
+            initargs=(tokenizer,),
+        ) as p:
             annotate_ = partial(
                 squad_convert_example_to_features,
                 max_seq_length=max_seq_length,
@@ -465,12 +495,17 @@ def squad_convert_examples_to_features(
             padding_strategy=padding_strategy,
             is_training=is_training,
         )
-        features = [annotate_(example) for example in tqdm(examples, disable=not tqdm_enabled)]
+        features = [
+            annotate_(example) for example in tqdm(examples, disable=not tqdm_enabled)
+        ]
     new_features = []
     unique_id = 1000000000
     example_index = 0
     for example_features in tqdm(
-        features, total=len(features), desc="add example index and unique id", disable=not tqdm_enabled
+        features,
+        total=len(features),
+        desc="add example index and unique id",
+        disable=not tqdm_enabled,
     ):
         if not example_features:
             continue
@@ -485,20 +520,35 @@ def squad_convert_examples_to_features(
 
     # Convert to Tensors and build dataset
     all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
-    all_attention_masks = torch.tensor([f.attention_mask for f in features], dtype=torch.long)
-    all_token_type_ids = torch.tensor([f.token_type_ids for f in features], dtype=torch.long)
+    all_attention_masks = torch.tensor(
+        [f.attention_mask for f in features], dtype=torch.long
+    )
+    all_token_type_ids = torch.tensor(
+        [f.token_type_ids for f in features], dtype=torch.long
+    )
     all_cls_index = torch.tensor([f.cls_index for f in features], dtype=torch.long)
     all_p_mask = torch.tensor([f.p_mask for f in features], dtype=torch.float)
-    all_is_impossible = torch.tensor([f.is_impossible for f in features], dtype=torch.float)
+    all_is_impossible = torch.tensor(
+        [f.is_impossible for f in features], dtype=torch.float
+    )
 
     if not is_training:
         all_feature_index = torch.arange(all_input_ids.size(0), dtype=torch.long)
         dataset = TensorDataset(
-            all_input_ids, all_attention_masks, all_token_type_ids, all_feature_index, all_cls_index, all_p_mask
+            all_input_ids,
+            all_attention_masks,
+            all_token_type_ids,
+            all_feature_index,
+            all_cls_index,
+            all_p_mask,
         )
     else:
-        all_start_positions = torch.tensor([f.start_position for f in features], dtype=torch.long)
-        all_end_positions = torch.tensor([f.end_position for f in features], dtype=torch.long)
+        all_start_positions = torch.tensor(
+            [f.start_position for f in features], dtype=torch.long
+        )
+        all_end_positions = torch.tensor(
+            [f.end_position for f in features], dtype=torch.long
+        )
         dataset = TensorDataset(
             all_input_ids,
             all_attention_masks,
@@ -573,7 +623,9 @@ def convert_examples_to_features(
         with Pool(args.process_count) as p:
             features = list(
                 tqdm(
-                    p.imap(convert_example_to_feature, example_rows, chunksize=chunksize),
+                    p.imap(
+                        convert_example_to_feature, example_rows, chunksize=chunksize
+                    ),
                     total=len(example_rows),
                     disable=args.silent,
                 )
@@ -612,7 +664,11 @@ def convert_examples_to_features(
                 else:
                     tok_end_position = len(all_doc_tokens) - 1
                 (tok_start_position, tok_end_position) = _improve_answer_span(
-                    all_doc_tokens, tok_start_position, tok_end_position, tokenizer, example.orig_answer_text,
+                    all_doc_tokens,
+                    tok_start_position,
+                    tok_end_position,
+                    tokenizer,
+                    example.orig_answer_text,
                 )
 
             # The -3 accounts for [CLS], [SEP] and [SEP]
@@ -621,7 +677,9 @@ def convert_examples_to_features(
             # We can have documents that are longer than the maximum sequence length.
             # To deal with this we do a sliding window approach, where we take chunks
             # of the up to our max length with a stride of `doc_stride`.
-            _DocSpan = collections.namedtuple("DocSpan", ["start", "length"])  # pylint: disable=invalid-name
+            _DocSpan = collections.namedtuple(
+                "DocSpan", ["start", "length"]
+            )  # pylint: disable=invalid-name
             doc_spans = []
             start_offset = 0
             while start_offset < len(all_doc_tokens):
@@ -666,9 +724,13 @@ def convert_examples_to_features(
                 # Paragraph
                 for i in range(doc_span.length):
                     split_token_index = doc_span.start + i
-                    token_to_orig_map[len(tokens)] = tok_to_orig_index[split_token_index]
+                    token_to_orig_map[len(tokens)] = tok_to_orig_index[
+                        split_token_index
+                    ]
 
-                    is_max_context = _check_is_max_context(doc_spans, doc_span_index, split_token_index)
+                    is_max_context = _check_is_max_context(
+                        doc_spans, doc_span_index, split_token_index
+                    )
                     token_is_max_context[len(tokens)] = is_max_context
                     tokens.append(all_doc_tokens[split_token_index])
                     if not sequence_a_is_doc:
@@ -726,7 +788,9 @@ def convert_examples_to_features(
                     doc_start = doc_span.start
                     doc_end = doc_span.start + doc_span.length - 1
                     out_of_span = False
-                    if not (tok_start_position >= doc_start and tok_end_position <= doc_end):
+                    if not (
+                        tok_start_position >= doc_start and tok_end_position <= doc_end
+                    ):
                         out_of_span = True
                     if out_of_span:
                         start_position = 0
@@ -768,7 +832,9 @@ def convert_examples_to_features(
     return features
 
 
-def _improve_answer_span(doc_tokens, input_start, input_end, tokenizer, orig_answer_text):
+def _improve_answer_span(
+    doc_tokens, input_start, input_end, tokenizer, orig_answer_text
+):
     """Returns tokenized answer spans that better match the annotated answer."""
 
     # The SQuAD annotations are character based. We first project them to
@@ -841,7 +907,9 @@ def _check_is_max_context(doc_spans, cur_span_index, position):
     return cur_span_index == best_span_index
 
 
-RawResult = collections.namedtuple("RawResult", ["unique_id", "start_logits", "end_logits"])
+RawResult = collections.namedtuple(
+    "RawResult", ["unique_id", "start_logits", "end_logits"]
+)
 
 
 def write_predictions(
@@ -871,7 +939,8 @@ def write_predictions(
         unique_id_to_result[result.unique_id] = result
 
     _PrelimPrediction = collections.namedtuple(  # pylint: disable=invalid-name
-        "PrelimPrediction", ["feature_index", "start_index", "end_index", "start_logit", "end_logit"],
+        "PrelimPrediction",
+        ["feature_index", "start_index", "end_index", "start_logit", "end_logit"],
     )
 
     all_predictions = collections.OrderedDict()
@@ -938,7 +1007,11 @@ def write_predictions(
                     end_logit=null_end_logit,
                 )
             )
-        prelim_predictions = sorted(prelim_predictions, key=lambda x: (x.start_logit + x.end_logit), reverse=True,)
+        prelim_predictions = sorted(
+            prelim_predictions,
+            key=lambda x: (x.start_logit + x.end_logit),
+            reverse=True,
+        )
 
         _NbestPrediction = collections.namedtuple(  # pylint: disable=invalid-name
             "NbestPrediction", ["text", "start_logit", "end_logit"]
@@ -966,7 +1039,9 @@ def write_predictions(
                 tok_text = " ".join(tok_text.split())
                 orig_text = " ".join(orig_tokens)
 
-                final_text = get_final_text(tok_text, orig_text, do_lower_case, verbose_logging)
+                final_text = get_final_text(
+                    tok_text, orig_text, do_lower_case, verbose_logging
+                )
                 if final_text in seen_predictions:
                     continue
 
@@ -975,16 +1050,28 @@ def write_predictions(
                 final_text = ""
                 seen_predictions[final_text] = True
 
-            nbest.append(_NbestPrediction(text=final_text, start_logit=pred.start_logit, end_logit=pred.end_logit,))
+            nbest.append(
+                _NbestPrediction(
+                    text=final_text,
+                    start_logit=pred.start_logit,
+                    end_logit=pred.end_logit,
+                )
+            )
         # if we didn't include the empty option in the n-best, include it
         if version_2_with_negative:
             if "" not in seen_predictions:
-                nbest.append(_NbestPrediction(text="", start_logit=null_start_logit, end_logit=null_end_logit))
+                nbest.append(
+                    _NbestPrediction(
+                        text="", start_logit=null_start_logit, end_logit=null_end_logit
+                    )
+                )
 
             # In very rare edge cases we could only have single null prediction.
             # So we just create a nonce prediction in this case to avoid failure.
             if len(nbest) == 1:
-                nbest.insert(0, _NbestPrediction(text="empty", start_logit=0.0, end_logit=0.0))
+                nbest.insert(
+                    0, _NbestPrediction(text="empty", start_logit=0.0, end_logit=0.0)
+                )
 
         # In very rare edge cases we could have no valid predictions. So we
         # just create a nonce prediction in this case to avoid failure.
@@ -1018,7 +1105,11 @@ def write_predictions(
             all_predictions[example.qas_id] = nbest_json[0]["text"]
         else:
             # predict "" iff the null score - the score of best non-null > threshold
-            score_diff = score_null - best_non_null_entry.start_logit - (best_non_null_entry.end_logit)
+            score_diff = (
+                score_null
+                - best_non_null_entry.start_logit
+                - (best_non_null_entry.end_logit)
+            )
             scores_diff_json[example.qas_id] = score_diff
             if score_diff > null_score_diff_threshold:
                 all_predictions[example.qas_id] = ""
@@ -1042,7 +1133,14 @@ def write_predictions(
 # For XLNet (and XLM which uses the same head)
 RawResultExtended = collections.namedtuple(
     "RawResultExtended",
-    ["unique_id", "start_top_log_probs", "start_top_index", "end_top_log_probs", "end_top_index", "cls_logits"],
+    [
+        "unique_id",
+        "start_top_log_probs",
+        "start_top_index",
+        "end_top_log_probs",
+        "end_top_index",
+        "cls_logits",
+    ],
 )
 
 
@@ -1062,12 +1160,13 @@ def write_predictions_extended(
     tokenizer,
     verbose_logging,
 ):
-    """ XLNet write prediction logic (more complex than Bert's).
-                    Write final predictions to the json file and log-odds of null if needed.
-                    Requires utils_squad_evaluate.py
+    """XLNet write prediction logic (more complex than Bert's).
+    Write final predictions to the json file and log-odds of null if needed.
+    Requires utils_squad_evaluate.py
     """
     _PrelimPrediction = collections.namedtuple(  # pylint: disable=invalid-name
-        "PrelimPrediction", ["feature_index", "start_index", "end_index", "start_log_prob", "end_log_prob"],
+        "PrelimPrediction",
+        ["feature_index", "start_index", "end_index", "start_log_prob", "end_log_prob"],
     )
 
     _NbestPrediction = collections.namedtuple(  # pylint: disable=invalid-name
@@ -1141,7 +1240,9 @@ def write_predictions_extended(
                     )
 
         prelim_predictions = sorted(
-            prelim_predictions, key=lambda x: (x.start_log_prob + x.end_log_prob), reverse=True,
+            prelim_predictions,
+            key=lambda x: (x.start_log_prob + x.end_log_prob),
+            reverse=True,
         )
 
         seen_predictions = {}
@@ -1181,13 +1282,19 @@ def write_predictions_extended(
             seen_predictions[final_text] = True
 
             nbest.append(
-                _NbestPrediction(text=final_text, start_log_prob=pred.start_log_prob, end_log_prob=pred.end_log_prob,)
+                _NbestPrediction(
+                    text=final_text,
+                    start_log_prob=pred.start_log_prob,
+                    end_log_prob=pred.end_log_prob,
+                )
             )
 
         # In very rare edge cases we could have no valid predictions. So we
         # just create a nonce prediction in this case to avoid failure.
         if not nbest:
-            nbest.append(_NbestPrediction(text="", start_log_prob=-1e6, end_log_prob=-1e6))
+            nbest.append(
+                _NbestPrediction(text="", start_log_prob=-1e6, end_log_prob=-1e6)
+            )
 
         total_scores = []
         best_non_null_entry = None
@@ -1238,7 +1345,9 @@ def write_predictions_extended(
     exact_raw, f1_raw = get_raw_scores(orig_data, all_predictions)
     out_eval = {}
 
-    find_all_best_thresh_v2(out_eval, all_predictions, exact_raw, f1_raw, scores_diff_json, qid_to_has_ans)
+    find_all_best_thresh_v2(
+        out_eval, all_predictions, exact_raw, f1_raw, scores_diff_json, qid_to_has_ans
+    )
 
     return all_predictions, all_nbest_json, scores_diff_json
 
@@ -1264,7 +1373,8 @@ def get_best_predictions(
         unique_id_to_result[result.unique_id] = result
 
     _PrelimPrediction = collections.namedtuple(  # pylint: disable=invalid-name
-        "PrelimPrediction", ["feature_index", "start_index", "end_index", "start_logit", "end_logit"],
+        "PrelimPrediction",
+        ["feature_index", "start_index", "end_index", "start_logit", "end_logit"],
     )
 
     all_predictions = collections.OrderedDict()
@@ -1331,7 +1441,11 @@ def get_best_predictions(
                     end_logit=null_end_logit,
                 )
             )
-        prelim_predictions = sorted(prelim_predictions, key=lambda x: (x.start_logit + x.end_logit), reverse=True,)
+        prelim_predictions = sorted(
+            prelim_predictions,
+            key=lambda x: (x.start_logit + x.end_logit),
+            reverse=True,
+        )
 
         _NbestPrediction = collections.namedtuple(  # pylint: disable=invalid-name
             "NbestPrediction", ["text", "start_logit", "end_logit"]
@@ -1359,7 +1473,9 @@ def get_best_predictions(
                 tok_text = " ".join(tok_text.split())
                 orig_text = " ".join(orig_tokens)
 
-                final_text = get_final_text(tok_text, orig_text, do_lower_case, verbose_logging)
+                final_text = get_final_text(
+                    tok_text, orig_text, do_lower_case, verbose_logging
+                )
                 if final_text in seen_predictions:
                     continue
 
@@ -1368,16 +1484,28 @@ def get_best_predictions(
                 final_text = ""
                 seen_predictions[final_text] = True
 
-            nbest.append(_NbestPrediction(text=final_text, start_logit=pred.start_logit, end_logit=pred.end_logit,))
+            nbest.append(
+                _NbestPrediction(
+                    text=final_text,
+                    start_logit=pred.start_logit,
+                    end_logit=pred.end_logit,
+                )
+            )
         # if we didn't include the empty option in the n-best, include it
         if version_2_with_negative:
             if "" not in seen_predictions:
-                nbest.append(_NbestPrediction(text="", start_logit=null_start_logit, end_logit=null_end_logit))
+                nbest.append(
+                    _NbestPrediction(
+                        text="", start_logit=null_start_logit, end_logit=null_end_logit
+                    )
+                )
 
             # In very rare edge cases we could only have single null prediction.
             # So we just create a nonce prediction in this case to avoid failure.
             if len(nbest) == 1:
-                nbest.insert(0, _NbestPrediction(text="empty", start_logit=0.0, end_logit=0.0))
+                nbest.insert(
+                    0, _NbestPrediction(text="empty", start_logit=0.0, end_logit=0.0)
+                )
 
         # In very rare edge cases we could have no valid predictions. So we
         # just create a nonce prediction in this case to avoid failure.
@@ -1411,7 +1539,11 @@ def get_best_predictions(
             all_predictions[example.qas_id] = nbest_json[0]["text"]
         else:
             # predict "" iff the null score - the score of best non-null > threshold
-            score_diff = score_null - best_non_null_entry.start_logit - (best_non_null_entry.end_logit)
+            score_diff = (
+                score_null
+                - best_non_null_entry.start_logit
+                - (best_non_null_entry.end_logit)
+            )
             scores_diff_json[example.qas_id] = score_diff
             if score_diff > null_score_diff_threshold:
                 all_predictions[example.qas_id] = ""
@@ -1442,12 +1574,13 @@ def get_best_predictions_extended(
     tokenizer,
     verbose_logging,
 ):
-    """ XLNet write prediction logic (more complex than Bert's).
-                    Write final predictions to the json file and log-odds of null if needed.
-                    Requires utils_squad_evaluate.py
+    """XLNet write prediction logic (more complex than Bert's).
+    Write final predictions to the json file and log-odds of null if needed.
+    Requires utils_squad_evaluate.py
     """
     _PrelimPrediction = collections.namedtuple(  # pylint: disable=invalid-name
-        "PrelimPrediction", ["feature_index", "start_index", "end_index", "start_log_prob", "end_log_prob"],
+        "PrelimPrediction",
+        ["feature_index", "start_index", "end_index", "start_log_prob", "end_log_prob"],
     )
 
     _NbestPrediction = collections.namedtuple(  # pylint: disable=invalid-name
@@ -1518,7 +1651,9 @@ def get_best_predictions_extended(
                     )
 
         prelim_predictions = sorted(
-            prelim_predictions, key=lambda x: (x.start_log_prob + x.end_log_prob), reverse=True,
+            prelim_predictions,
+            key=lambda x: (x.start_log_prob + x.end_log_prob),
+            reverse=True,
         )
 
         seen_predictions = {}
@@ -1553,7 +1688,9 @@ def get_best_predictions_extended(
             if isinstance(tokenizer, XLMTokenizer):
                 final_text = get_final_text(tok_text, orig_text, verbose_logging)
             else:
-                final_text = get_final_text(tok_text, orig_text, tokenizer.do_lower_case, verbose_logging)
+                final_text = get_final_text(
+                    tok_text, orig_text, tokenizer.do_lower_case, verbose_logging
+                )
 
             if final_text in seen_predictions:
                 continue
@@ -1561,13 +1698,19 @@ def get_best_predictions_extended(
             seen_predictions[final_text] = True
 
             nbest.append(
-                _NbestPrediction(text=final_text, start_log_prob=pred.start_log_prob, end_log_prob=pred.end_log_prob,)
+                _NbestPrediction(
+                    text=final_text,
+                    start_log_prob=pred.start_log_prob,
+                    end_log_prob=pred.end_log_prob,
+                )
             )
 
         # In very rare edge cases we could have no valid predictions. So we
         # just create a nonce prediction in this case to avoid failure.
         if not nbest:
-            nbest.append(_NbestPrediction(text="", start_log_prob=-1e6, end_log_prob=-1e6))
+            nbest.append(
+                _NbestPrediction(text="", start_log_prob=-1e6, end_log_prob=-1e6)
+            )
 
         total_scores = []
         best_non_null_entry = None
@@ -1609,9 +1752,15 @@ def get_best_predictions_extended(
     return all_best
 
 
-def find_all_best_thresh_v2(main_eval, preds, exact_raw, f1_raw, na_probs, qid_to_has_ans):
-    best_exact, exact_thresh, has_ans_exact = find_best_thresh_v2(preds, exact_raw, na_probs, qid_to_has_ans)
-    best_f1, f1_thresh, has_ans_f1 = find_best_thresh_v2(preds, f1_raw, na_probs, qid_to_has_ans)
+def find_all_best_thresh_v2(
+    main_eval, preds, exact_raw, f1_raw, na_probs, qid_to_has_ans
+):
+    best_exact, exact_thresh, has_ans_exact = find_best_thresh_v2(
+        preds, exact_raw, na_probs, qid_to_has_ans
+    )
+    best_f1, f1_thresh, has_ans_f1 = find_best_thresh_v2(
+        preds, f1_raw, na_probs, qid_to_has_ans
+    )
     main_eval["best_exact"] = best_exact
     main_eval["best_exact_thresh"] = exact_thresh
     main_eval["best_f1"] = best_f1
@@ -1672,7 +1821,9 @@ def get_raw_scores(dataset, preds):
     for p in dataset:
         for qa in p["qas"]:
             qid = qa["id"]
-            gold_answers = [a["text"] for a in qa["answers"] if normalize_answer(a["text"])]
+            gold_answers = [
+                a["text"] for a in qa["answers"] if normalize_answer(a["text"])
+            ]
             if not gold_answers:
                 # For unanswerable questions, only correct answer is empty string
                 gold_answers = [""]
@@ -1792,7 +1943,9 @@ def get_final_text(pred_text, orig_text, do_lower_case, verbose_logging=False):
     if len(orig_ns_text) != len(tok_ns_text):
         if verbose_logging:
             logger.info(
-                "Length not equal after stripping spaces: '%s' vs '%s'", orig_ns_text, tok_ns_text,
+                "Length not equal after stripping spaces: '%s' vs '%s'",
+                orig_ns_text,
+                tok_ns_text,
             )
         return orig_text
 
@@ -1897,14 +2050,6 @@ class LazyQuestionAnsweringDataset(Dataset):
 
         return counter
 
-    # @staticmethod
-    # def _get_n_lines(data_file):
-    #     with open(data_file, encoding="utf-8") as f:
-    #         for line_idx, _ in enumerate(f, 1):
-    #             pass
-
-    #     return line_idx
-
     def __getitem__(self, idx):
         if idx == 0:
             idx = 1
@@ -1912,7 +2057,11 @@ class LazyQuestionAnsweringDataset(Dataset):
         qa_sample = json.loads(line)
         example = get_examples([qa_sample])[0]
         f = squad_convert_example_to_features(
-            example, self.args.max_seq_length, self.args.doc_stride, self.args.max_query_length, True
+            example,
+            self.args.max_seq_length,
+            self.args.doc_stride,
+            self.args.max_query_length,
+            True,
         )[0]
 
         return (
@@ -1928,3 +2077,143 @@ class LazyQuestionAnsweringDataset(Dataset):
 
     def __len__(self):
         return self.num_entries
+
+
+def preprocess_batch_for_hf_dataset(
+    examples, max_seq_length, doc_stride, max_query_length, is_training
+):
+    qas_id_batch = examples["qas_id"]
+    question_text_batch = examples["question_text"]
+    context_text_batch = examples["context_text"]
+    answer_text_batch = examples["answer_text"]
+    start_position_character_batch = examples["start_position_character"]
+    is_impossible_batch = examples["is_impossible"]
+    answers_batch = examples["answers"]
+
+    input_ids_list = []
+    attention_mask_list = []
+    token_type_ids_list = []
+    cls_index_list = []
+    start_position_list = []
+    end_position_list = []
+    p_mask_list = []
+
+    for (
+        qas_id,
+        question_text,
+        context_text,
+        answer_text,
+        start_position_character,
+        is_impossible,
+        answers,
+    ) in zip(
+        qas_id_batch,
+        question_text_batch,
+        context_text_batch,
+        answer_text_batch,
+        start_position_character_batch,
+        is_impossible_batch,
+        answers_batch,
+    ):
+        example = SquadExample(
+            qas_id=qas_id,
+            question_text=question_text,
+            context_text=context_text,
+            answer_text=answer_text if answer_text else None,
+            start_position_character=start_position_character
+            if start_position_character != -1
+            else None,
+            title=None,
+            is_impossible=is_impossible,
+            answers=answers,
+        )
+
+        f = squad_convert_example_to_features(
+            example,
+            max_seq_length,
+            doc_stride,
+            max_query_length,
+            padding_strategy="max_length",
+            is_training=is_training,
+        )
+
+        if f:
+            f = f[0]
+        else:
+            raise ValueError(
+                "Make sure the data file does not contain errors. Could not find answer <{}> in <{}>".format(
+                    answer_text, context_text
+                )
+            )
+
+        input_ids_list.append(f.input_ids)
+        attention_mask_list.append(f.attention_mask)
+        token_type_ids_list.append(f.token_type_ids)
+        cls_index_list.append(f.cls_index)
+        start_position_list.append(f.start_position)
+        end_position_list.append(f.end_position)
+        p_mask_list.append(f.p_mask)
+
+    features = {
+        "input_ids": input_ids_list,
+        "attention_mask": attention_mask_list,
+        "token_type_ids": token_type_ids_list,
+        "cls_index": cls_index_list,
+        "start_positions": start_position_list,
+        "end_positions": end_position_list,
+        "p_mask": p_mask_list,
+    }
+
+    return features
+
+
+def load_hf_dataset(data, tokenizer, args, is_training):
+    if isinstance(data, str):
+        dataset = load_dataset(
+            os.path.join(
+                os.path.dirname(os.path.abspath(__file__)), "qa_dataset_loading_script"
+            ),
+            data_files=data,
+            is_training=is_training,
+            download_mode="force_redownload"
+            if args.reprocess_input_data
+            else "reuse_dataset_if_exists",
+        )
+    else:
+        raise TypeError(
+            "{} is not a path to a json file. The input must be a json file for QAModel.".format(
+                data
+            )
+        )
+
+    squad_convert_example_to_features_init(tokenizer)
+
+    dataset = dataset.map(
+        lambda x: preprocess_batch_for_hf_dataset(
+            x,
+            args.max_seq_length,
+            args.doc_stride,
+            args.max_query_length,
+            is_training=is_training,
+        ),
+        batched=True,
+    )
+
+    dataset.set_format(
+        type="pt",
+        columns=[
+            "input_ids",
+            "attention_mask",
+            "token_type_ids",
+            "cls_index",
+            "start_positions",
+            "end_positions",
+            "p_mask",
+        ],
+    )
+
+    if isinstance(data, str):
+        # This is not necessarily a train dataset. The datasets library insists on calling it train.
+        return dataset["train"]
+    else:
+        return dataset
