@@ -707,6 +707,7 @@ class Seq2SeqModel:
             )
             wandb.run._label(repo="simpletransformers")
             wandb.watch(self.model)
+            self.wandb_run_id = wandb.run.id
 
         if args.fp16:
             from torch.cuda import amp
@@ -1491,7 +1492,11 @@ class Seq2SeqModel:
                     )
                 else:
                     return Seq2SeqDataset(
-                        encoder_tokenizer, decoder_tokenizer, self.args, data, mode,
+                        encoder_tokenizer,
+                        decoder_tokenizer,
+                        self.args,
+                        data,
+                        mode,
                     )
 
     def _create_training_progress_scores(self, **kwargs):
