@@ -27,8 +27,10 @@ from transformers import (
 logger = logging.getLogger(__name__)
 
 if transformers.__version__ < "4.2.0":
-    shift_tokens_right = lambda input_ids, pad_token_id, decoder_start_token_id: _shift_tokens_right(
-        input_ids, pad_token_id
+    shift_tokens_right = (
+        lambda input_ids, pad_token_id, decoder_start_token_id: _shift_tokens_right(
+            input_ids, pad_token_id
+        )
     )
 else:
     shift_tokens_right = _shift_tokens_right
